import { None, Some } from 'option-type';

export function query(context, selectors, klass) {
  klass = klass || HTMLElement;
  return querySelector(context, selectors, klass).expect('Element not found: <' + klass.name + '> ' + selectors);
}

export function querySelector(context, selectors, klass) {
  klass = klass || HTMLElement;
  var el = context.querySelector(selectors);
  return el instanceof klass ? Some(el) : None;
}

export function querySelectorAll(context, selectors, klass) {
  klass = klass || HTMLElement;
  var found = [];
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = context.querySelectorAll(selectors)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var el = _step.value;

      if (el instanceof klass) {
        found.push(el);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return found;
}

export function closest(element, selectors, klass) {
  klass = klass || HTMLElement;
  var ancestor = element.closest(selectors);
  return ancestor instanceof klass ? Some(ancestor) : None;
}

export function find(selectors, klass) {
  klass = klass || HTMLElement;
  return function descendant(el) {
    return querySelector(el, selectors, klass);
  };
}

export function parent(selectors, klass) {
  klass = klass || HTMLElement;
  return function ancestor(el) {
    var node = el.parentElement;
    if (!node) return None;
    if (node instanceof klass && node.matches(selectors)) {
      return Some(node);
    }
    return ancestor(node);
  };
}

export function addClass() {
  for (var _len = arguments.length, names = Array(_len), _key = 0; _key < _len; _key++) {
    names[_key] = arguments[_key];
  }

  return function add(el) {
    var _el$classList;

    (_el$classList = el.classList).add.apply(_el$classList, names);
    return Some(el);
  };
}

export function removeClass() {
  for (var _len2 = arguments.length, names = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    names[_key2] = arguments[_key2];
  }

  return function remove(el) {
    var _el$classList2;

    (_el$classList2 = el.classList).remove.apply(_el$classList2, names);
    return Some(el);
  };
}

export function toggleClass(name, force) {
  return function toggle(el) {
    el.classList.toggle(name, force);
    return Some(el);
  };
}

export function prev(selectors, klass) {
  klass = klass || HTMLElement;
  return function previousElement(el) {
    var node = el.previousElementSibling;
    if (!node) return None;
    if (node instanceof klass && node.matches(selectors)) {
      return Some(node);
    }
    return previousElement(node);
  };
}

export function next(selectors, klass) {
  klass = klass || HTMLElement;
  return function nextElement(el) {
    var node = el.nextElementSibling;
    if (!node) return None;
    if (node instanceof klass && node.matches(selectors)) {
      return Some(node);
    }
    return nextElement(node);
  };
}

export function getAttribute(name) {
  return function get(el) {
    var value = el.getAttribute(name);
    return value == null ? None : Some(value);
  };
}

export function setAttribute(name, value) {
  return function set(el) {
    el.setAttribute(name, value);
    return Some(el);
  };
}

export function getValue(el) {
  return el.value ? Some(el.value) : None;
}

export function setValue(value) {
  return function set(el) {
    el.value = value;
    return Some(el);
  };
}

export function setText(value) {
  return function set(el) {
    el.textContent = value;
    return Some(el);
  };
}

export function append() {
  for (var _len3 = arguments.length, nodes = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    nodes[_key3] = arguments[_key3];
  }

  return function appendTo(el) {
    el.append.apply(el, nodes);
    return Some(el);
  };
}

export function prepend() {
  for (var _len4 = arguments.length, nodes = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
    nodes[_key4] = arguments[_key4];
  }

  return function prependTo(el) {
    el.prepend.apply(el, nodes);
    return Some(el);
  };
}

export function after() {
  for (var _len5 = arguments.length, nodes = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
    nodes[_key5] = arguments[_key5];
  }

  return function addAfter(el) {
    el.after.apply(el, nodes);
    return Some(el);
  };
}

export function before() {
  for (var _len6 = arguments.length, nodes = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
    nodes[_key6] = arguments[_key6];
  }

  return function addBefore(el) {
    el.before.apply(el, nodes);
    return Some(el);
  };
}

export function replaceWith() {
  for (var _len7 = arguments.length, nodes = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
    nodes[_key7] = arguments[_key7];
  }

  return function replace(el) {
    el.replaceWith.apply(el, nodes);
    return Some(el);
  };
}

export function remove(el) {
  el.remove();
  return Some(el);
}

export function namedItem(name, klass) {
  klass = klass || HTMLInputElement;
  return function named(form) {
    var el = form.elements.namedItem(name);
    return el instanceof klass ? Some(el) : None;
  };
}
