(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'option-type'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('option-type'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.optionType);
    global.index = mod.exports;
  }
})(this, function (exports, _optionType) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.query = query;
  exports.querySelector = querySelector;
  exports.querySelectorAll = querySelectorAll;
  exports.closest = closest;
  exports.find = find;
  exports.parent = parent;
  exports.addClass = addClass;
  exports.removeClass = removeClass;
  exports.toggleClass = toggleClass;
  exports.prev = prev;
  exports.next = next;
  exports.getAttribute = getAttribute;
  exports.setAttribute = setAttribute;
  exports.getValue = getValue;
  exports.setValue = setValue;
  exports.setText = setText;
  exports.append = append;
  exports.prepend = prepend;
  exports.after = after;
  exports.before = before;
  exports.replaceWith = replaceWith;
  exports.remove = remove;
  exports.namedItem = namedItem;
  function query(context, selectors, klass) {
    klass = klass || HTMLElement;
    return querySelector(context, selectors, klass).expect('Element not found: <' + klass.name + '> ' + selectors);
  }

  function querySelector(context, selectors, klass) {
    klass = klass || HTMLElement;
    var el = context.querySelector(selectors);
    return el instanceof klass ? (0, _optionType.Some)(el) : _optionType.None;
  }

  function querySelectorAll(context, selectors, klass) {
    klass = klass || HTMLElement;
    var found = [];
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = context.querySelectorAll(selectors)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var el = _step.value;

        if (el instanceof klass) {
          found.push(el);
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return found;
  }

  function closest(element, selectors, klass) {
    klass = klass || HTMLElement;
    var ancestor = element.closest(selectors);
    return ancestor instanceof klass ? (0, _optionType.Some)(ancestor) : _optionType.None;
  }

  function find(selectors, klass) {
    klass = klass || HTMLElement;
    return function descendant(el) {
      return querySelector(el, selectors, klass);
    };
  }

  function parent(selectors, klass) {
    klass = klass || HTMLElement;
    return function ancestor(el) {
      var node = el.parentElement;
      if (!node) return _optionType.None;
      if (node instanceof klass && node.matches(selectors)) {
        return (0, _optionType.Some)(node);
      }
      return ancestor(node);
    };
  }

  function addClass() {
    for (var _len = arguments.length, names = Array(_len), _key = 0; _key < _len; _key++) {
      names[_key] = arguments[_key];
    }

    return function add(el) {
      var _el$classList;

      (_el$classList = el.classList).add.apply(_el$classList, names);
      return (0, _optionType.Some)(el);
    };
  }

  function removeClass() {
    for (var _len2 = arguments.length, names = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      names[_key2] = arguments[_key2];
    }

    return function remove(el) {
      var _el$classList2;

      (_el$classList2 = el.classList).remove.apply(_el$classList2, names);
      return (0, _optionType.Some)(el);
    };
  }

  function toggleClass(name, force) {
    return function toggle(el) {
      el.classList.toggle(name, force);
      return (0, _optionType.Some)(el);
    };
  }

  function prev(selectors, klass) {
    klass = klass || HTMLElement;
    return function previousElement(el) {
      var node = el.previousElementSibling;
      if (!node) return _optionType.None;
      if (node instanceof klass && node.matches(selectors)) {
        return (0, _optionType.Some)(node);
      }
      return previousElement(node);
    };
  }

  function next(selectors, klass) {
    klass = klass || HTMLElement;
    return function nextElement(el) {
      var node = el.nextElementSibling;
      if (!node) return _optionType.None;
      if (node instanceof klass && node.matches(selectors)) {
        return (0, _optionType.Some)(node);
      }
      return nextElement(node);
    };
  }

  function getAttribute(name) {
    return function get(el) {
      var value = el.getAttribute(name);
      return value == null ? _optionType.None : (0, _optionType.Some)(value);
    };
  }

  function setAttribute(name, value) {
    return function set(el) {
      el.setAttribute(name, value);
      return (0, _optionType.Some)(el);
    };
  }

  function getValue(el) {
    return el.value ? (0, _optionType.Some)(el.value) : _optionType.None;
  }

  function setValue(value) {
    return function set(el) {
      el.value = value;
      return (0, _optionType.Some)(el);
    };
  }

  function setText(value) {
    return function set(el) {
      el.textContent = value;
      return (0, _optionType.Some)(el);
    };
  }

  function append() {
    for (var _len3 = arguments.length, nodes = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      nodes[_key3] = arguments[_key3];
    }

    return function appendTo(el) {
      el.append.apply(el, nodes);
      return (0, _optionType.Some)(el);
    };
  }

  function prepend() {
    for (var _len4 = arguments.length, nodes = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
      nodes[_key4] = arguments[_key4];
    }

    return function prependTo(el) {
      el.prepend.apply(el, nodes);
      return (0, _optionType.Some)(el);
    };
  }

  function after() {
    for (var _len5 = arguments.length, nodes = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
      nodes[_key5] = arguments[_key5];
    }

    return function addAfter(el) {
      el.after.apply(el, nodes);
      return (0, _optionType.Some)(el);
    };
  }

  function before() {
    for (var _len6 = arguments.length, nodes = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
      nodes[_key6] = arguments[_key6];
    }

    return function addBefore(el) {
      el.before.apply(el, nodes);
      return (0, _optionType.Some)(el);
    };
  }

  function replaceWith() {
    for (var _len7 = arguments.length, nodes = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
      nodes[_key7] = arguments[_key7];
    }

    return function replace(el) {
      el.replaceWith.apply(el, nodes);
      return (0, _optionType.Some)(el);
    };
  }

  function remove(el) {
    el.remove();
    return (0, _optionType.Some)(el);
  }

  function namedItem(name, klass) {
    klass = klass || HTMLInputElement;
    return function named(form) {
      var el = form.elements.namedItem(name);
      return el instanceof klass ? (0, _optionType.Some)(el) : _optionType.None;
    };
  }
});
